<?php
/**
 * Created by PhpStorm.
 * User: Mead
 * Date: 2019/10/8
 * Time: 3:41 PM
 */

namespace App\Handlers;

use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Redis;

class BikeStatusInfoSyncHandler
{
    //用户
    const ROLE_USER = 'user';
    //运维人员
    const ROLE_WORKER = 'worker';
    //系统本身
    const ROLE_SERVER = 'server';
    //绑定人员
    const ROLE_BIND = 'bind';

    private static $redis = '';

    //还车时过期时间
    const CLOSE_BIKE_TTL = 30;

    //正常未骑行车的缓存标识
    const REDIS_BIKE_LOCATION_TAG = 'bike_locations';
    const REDIS_RIDE_BIKE_ORDERS_TAG = 'ride_orders:';

    public function __construct()
    {
        self::$redis = Redis::connection();
    }

    /**
     * 车辆处于骑行状态 (用户订单时（data: 订单id[id],区域id[area_id],车辆id[bike_id]）)
     * @param $bike_no
     * @param array $data
     * User: Mead
     */
    public function toBikeRideStatus($role, $bike_no, $data = [])
    {
        $data['role'] = $role;
        //临时锁车状态
        $data['is_temporary_close'] = 0;
        //是否超出运营区域
        $data['is_out_area_lost_electric'] = 1;
        //是否低电关电车
        $data['is_low_electric_close_bike'] = 1;
        self::$redis->set(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no, serialize($data));
        //删除未骑行状态
        self::$redis->zrem(self::REDIS_BIKE_LOCATION_TAG, $bike_no);
    }

    /**
     * 车辆处于等待骑行状态
     * @param $bike_no
     * User: Mead
     */
    public function toBikeWaitRideStatus($bike_no, $lng = 0, $lat = 0)
    {
//        self::$redis->hdel(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no);
        self::$redis->expire(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no, self::CLOSE_BIKE_TTL);
        self::$redis->geoadd(self::REDIS_BIKE_LOCATION_TAG, $lng, $lat, $bike_no);
    }

    /**
     * 改变车为订单为临时停车
     * @param $bike_no
     * User: Mead
     */
    public function toBikeTemporaryWaitRideStatus($bike_no)
    {
        $data = $this->getRideBikeOrderInfo($bike_no);
        $data['is_temporary_close'] = 1;
        self::$redis->set(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no, serialize($data));
    }

    /**
     * 改变车为订单为临时停车
     * @param $bike_no
     * User: Mead
     */
    public function toBikeTemporaryRideStatus($bike_no)
    {
        $data = $this->getRideBikeOrderInfo($bike_no);
        $data['is_temporary_close'] = 0;
        self::$redis->set(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no, serialize($data));
    }

    /**
     * 改变车为订单为开车(租车)
     * @param $bike_no
     * User: Mead
     */
    public function toBikeRentRideStatus($bike_no)
    {
        $data = $this->getRideBikeOrderInfo($bike_no);
        if (!$data) {
            $order = DB::table('rent_orders')->where('bike_no', $bike_no)->orderByDesc('id')->first();
            $data = [
                'id' => $order->id,
                'bike_id' => $order->bike_id,
                'area_id' => $order->area_id,
                'is_rent' => 1,
                'role' => 'user'
            ];
            //是否超出运营区域
            $data['is_out_area_lost_electric'] = 1;
            //是否低电关电车
            $data['is_low_electric_close_bike'] = 1;
        }
        $data['is_close_bike'] = 0;
        self::$redis->set(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no, serialize($data));
    }

    /**
     * 改变车为订单为停车(租车)
     * @param $bike_no
     * User: Mead
     */
    public function toBikeRentWaitRideStatus($bike_no)
    {
        $data = $this->getRideBikeOrderInfo($bike_no);
        if (!$data) {
            $order = DB::table('rent_orders')->where('bike_no', $bike_no)->orderByDesc('id')->first();
            $data = [
                'id' => $order->id,
                'bike_id' => $order->bike_id,
                'area_id' => $order->area_id,
                'is_rent' => 1,
                'role' => 'user'
            ];
            //是否超出运营区域
            $data['is_out_area_lost_electric'] = 1;
            //是否低电关电车
            $data['is_low_electric_close_bike'] = 1;
        }
        $data['is_close_bike'] = 1;
        self::$redis->set(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no, serialize($data));
    }

    /**
     * 车辆上线
     * @param $bike_no
     * @param int $lat
     * @param int $lng
     * User: Mead
     */
    public function toBikeOnLineStatus($bike_no, $lng = 0, $lat = 0)
    {
//        self::$redis->hdel(self::REDIS_RIDE_BIKE_ORDERS_TAG, $bike_no);
        self::$redis->del([self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no]);
        self::$redis->geoadd(self::REDIS_BIKE_LOCATION_TAG, $lng, $lat, $bike_no);
    }

    /**
     * 车辆离线
     * @param $bike_no
     * User: Mead
     */
    public function toBikeOffLineStatus($bike_no)
    {
        //删除未骑行状态
        self::$redis->zrem(self::REDIS_BIKE_LOCATION_TAG, $bike_no);
    }

    /**
     * 获取订单信息
     * @param $bike_no
     * @return mixed
     * User: Mead
     */
    public function getRideBikeOrderInfo($bike_no)
    {
        $order = self::$redis->get(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no);
        if (!$order) return false;
        return unserialize($order);
    }
}
