<?php


namespace App\Http\Controllers\V1;


use App\Http\Requests\AnnouncementRequest;
use App\Http\Requests\AnnouncementsRequest;
use App\Repositories\AnnouncementRepository;
use App\Transformers\AnnouncementTransformer;
use App\Transformers\AnnouncementsTransformer;

/**
 * 公告管理模块
 * Class AnnouncementController
 * @package App\Http\Controllers\V1
 */
class AnnouncementController extends BaseController
{
    /**
     * 公告列表
     * @param AnnouncementsRequest $request
     * @param AnnouncementRepository $advertisementRepository
     * @return \Dingo\Api\Http\Response
     * User: FX
     */
    public function index(AnnouncementsRequest $request, AnnouncementRepository $repository)
    {
        try {
            $area_id = $request->get('area_id', 0);
            $ans = $repository->byAreaIdGetAns($area_id);
            return $this->response->paginator($ans, AnnouncementsTransformer::class);
        } catch (\Exception $exception) {
            $this->errorNoValidation($exception->getMessage());
        }
    }

    /**
     * 公告详情
     * @param AnnouncementRequest $request
     * @param AnnouncementRepository $advertisementRepository
     * @return \Dingo\Api\Http\Response
     * User: FX
     */
    public function show(AnnouncementRequest $request, AnnouncementRepository $repository)
    {
        try {
            $id = $request->get('id');
            $ads = $repository->byIdGetModel($id);
            return $this->response->item($ads, AnnouncementTransformer::class);
        } catch (\Exception $exception) {
            $this->errorNoValidation($exception->getMessage());
        }
    }

    /**
     * 首页显示公告
     *
     * */
    public function indexShow(AnnouncementsRequest $announcementsRequest, AnnouncementRepository $repository)
    {
        try {
            $area_id = $announcementsRequest->get('area_id', 0);
            $ans = $repository->byAreaIdGetLastAn($area_id);
            if (empty($ans)) return $this->response->array([]);
            return $this->response->item($ans, AnnouncementTransformer::class);
        } catch (\Exception $exception) {
            $this->errorNoValidation($exception->getMessage());
        }
    }
}
