<?php
/**
 * Created by PhpStorm.
 * User: Mead
 * Date: 2019/8/6
 * Time: 9:16 PM
 */

namespace App\Repositories;


use App\Maps\CacheMap;
use App\Models\AreaSetting;
use Carbon\Carbon;
use Illuminate\Support\Facades\Cache;

class AreaSettingRepository extends BaseRepository
{

    public function __construct(AreaSetting $model)
    {
        $this->model = $model;
    }


    /**
     * 根据area_id 获取区域配置
     * @param $area_id
     * @return mixed
     * User: Mead
     */
    public function byAreaId($area_id)
    {
        return $this->model->where('area_id', $area_id)->first();
    }

    /**
     * 根据area_id获取是否全区域可以还车
     * @param $area_id
     * @return int
     */
    public function byAreaIdGetIsWholeAreaHuanche($area_id): int
    {
        return Cache::remember(CacheMap::BY_AREA_ID_GET_IS_WHOLE_AREA_HUANCHE . $area_id, Carbon::now()->addSeconds(CacheMap::CACHE_TIME), function () use ($area_id) {
            return $this->model->where('area_id', $area_id)->value('is_whole_area_huanche');
        });
    }

    /**
     * 根据area_id获取是否全区域可以还车
     * @param $area_id
     * @return int
     */
    public function byAreaIdGetFreeMinute($area_id): int
    {
        return Cache::remember(CacheMap::BY_AREA_ID_GET_FREE_MINUTE . $area_id, Carbon::now()->addSeconds(CacheMap::CACHE_TIME), function () use ($area_id) {
            return $this->model->where('area_id', $area_id)->value('many_minute_free');
        });
    }

    /**
     * 根据area_id获取是否全区域可以还车
     * @param $area_id
     * @return int
     */
    public function byAreaIdGetBatteryPower($area_id): int
    {
        return Cache::remember(CacheMap::BY_AREA_ID_GET_BATTERY_POWER . $area_id, Carbon::now()->addSeconds(CacheMap::CACHE_TIME), function () use ($area_id) {
            return $this->model->where('area_id', $area_id)->value('power_lower');
        });
    }

}
